terraform {
  required_providers {
    aws = {
        source  = "hashicorp/aws"
        version = "4.63.0"
      }
  }
}

provider "aws" {
  region = "eu-west-1"
  profile = "default"
}

variable "bucket" {
  type = string
  default = "bucket-name"
}

variable "env-ver" {
  type = string
}

data "aws_elastic_beanstalk_solution_stack" "Nodejs" {
  most_recent = true
 
  name_regex  = "^64bit Amazon Linux 2 (.*) Node.js (.*)$"
}


resource "aws_iam_instance_profile" "elasticbeanstalk-instance-profile" {
  name = "elasticbeanstalk-instance-profile"
  role = "AdminInstance"

}

resource "aws_elastic_beanstalk_application" "tftest" {
  name        = "tf-test-name"
}

resource "aws_elastic_beanstalk_application_version" "default" {
  name        = "tf-test-version"
  application = aws_elastic_beanstalk_application.tftest.name
  description = "application version created by terraform"
  bucket      = aws_s3_bucket.default.id
  key         = aws_s3_object.default.id
}

resource "aws_elastic_beanstalk_environment" "env" {
  name         = "Environment"
  application  = aws_elastic_beanstalk_application.tftest.name

  solution_stack_name = data.aws_elastic_beanstalk_solution_stack.Nodejs.name

  setting {
    namespace = "aws:elasticbeanstalk:application:environment"
    name      = "VERSION"
    value     = var.env-ver
  }
  
  setting {
    namespace = "aws:autoscaling:launchconfiguration"
    name      = "IamInstanceProfile"
    value     = aws_iam_instance_profile.elasticbeanstalk-instance-profile.name
  }

  setting {
    namespace = "aws:autoscaling:updatepolicy:rollingupdate"
    name      = "RollingUpdateType"
    value     = "Immutable"
  }
  setting {
    namespace = "aws:elasticbeanstalk:command"
    name      = "DeploymentPolicy"
    value     = "Immutable"
  }

  tags = {
    version = var.env-ver
  }
}

resource "aws_s3_bucket" "default" {
  bucket = var.bucket 
}

resource "aws_s3_object" "default" {
  bucket = aws_s3_bucket.default.id
  key    = "beanstalk/nodejs.zip"
  source = "nodejs.zip"
  etag = filemd5("nodejs.zip")
}


